import 'dart:async';

import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../model/java_script_channel.dart';
import '../widgets/scaffold_webview.dart';

sealed class CLXWebViewBaseWebViewModel extends ChangeNotifier {
  CLXWebViewBaseWebViewModel.loadRequest({required Uri requestedUrl}) {
    _controller = WebViewController();
    _controller.setJavaScriptMode(JavaScriptMode.unrestricted);
    _controller.loadRequest(requestedUrl);
  }

  CLXWebViewBaseWebViewModel.loadHtmlString(
      {required String html, String? baseUrl}) {
    _controller = WebViewController();
    _controller.setJavaScriptMode(JavaScriptMode.unrestricted);
    _controller.loadHtmlString(html, baseUrl: baseUrl);
  }

  CLXWebViewBaseWebViewModel.loadFile({required String absoluteFilePath}) {
    _controller = WebViewController();
    _controller.setJavaScriptMode(JavaScriptMode.unrestricted);
    _controller.loadFile(absoluteFilePath);
  }

  /// 外部访问WebViewController的接口
  late WebViewController _controller;
  WebViewController get controller => _controller;

  /// 外部访问JavaScriptChannel的接口
  final List<JavaScriptChannel> _javaScriptChannels = [];
  List<JavaScriptChannel>? get javaScriptChannels => _javaScriptChannels;
  Future<bool> setJavaScriptChannels(JavaScriptChannel channel) async {
    if (_javaScriptChannels
        .where((element) => element.name == channel.name)
        .isNotEmpty) {
      return Future.value(false);
    }
    _javaScriptChannels.add(channel);
    controller.addJavaScriptChannel(channel.name,
        onMessageReceived: channel.channelCallback.run);
    controller.reload();
    return Future.value(true);
  }

  Future<bool> removeJavaScriptChannel(String name) async {
    if (_javaScriptChannels.where((element) => element.name == name).isEmpty) {
      return Future.value(false);
    }
    _javaScriptChannels.removeWhere((element) => element.name == name);
    controller.removeJavaScriptChannel(name);
    controller.reload();
    return Future.value(true);
  }

  Future<bool> showJavaScriptLog(
      void Function(JavaScriptConsoleMessage message) onConsoleMessage) async {
    controller.setOnConsoleMessage(onConsoleMessage);
    return Future.value(true);
  }
}

class ListenableWebViewViewModel extends CLXWebViewBaseWebViewModel {
  ListenableWebViewViewModel.loadRequest({required super.requestedUrl})
      : super.loadRequest();

  ListenableWebViewViewModel.loadHtmlString(
      {required super.html, super.baseUrl})
      : super.loadHtmlString();

  ListenableWebViewViewModel.loadFile({required super.absoluteFilePath})
      : super.loadFile();
}

class ScalloldWebViewViewModel extends CLXWebViewBaseWebViewModel {
  ScalloldWebViewViewModel.loadRequest(
      {required super.requestedUrl, String? title, TextStyle? appBarTextStyle})
      : super.loadRequest() {
    _title = title;
    _appBarTextStyle = appBarTextStyle;
  }

  ScalloldWebViewViewModel.loadHtmlString(
      {required super.html,
      super.baseUrl,
      String? title,
      TextStyle? appBarTextStyle})
      : super.loadHtmlString() {
    _title = title;
    _appBarTextStyle = appBarTextStyle;
  }

  ScalloldWebViewViewModel.loadFile({
    required super.absoluteFilePath,
    String? title,
    TextStyle? appBarTextStyle,
  }) : super.loadFile() {
    _title = title;
    _appBarTextStyle = appBarTextStyle;
  }

  /// 内部访问WebView的接口
  late ScaffoldWebViewState _customWebViewState;
  void bindState(ScaffoldWebViewState state) {
    _customWebViewState = state;
  }

  /// 内部访问AppBar的title接口
  String? _title;
  AppBar? get appBar => _title == null
      ? null
      : _isHideNavigationBar
          ? null
          : AppBar(
              title: ListenableBuilder(
                listenable: this,
                builder: (context, child) {
                  return Text(_title!, style: _appBarTextStyle);
                },
              ),
            );
  Future<bool> setTitle(String title) async {
    if (!_customWebViewState.mounted) {
      return Future.value(false);
    } else {
      _title = title;
      notifyListeners();
      return Future.value(true);
    }
  }

  /// 外部访问AppBar的title样式接口
  TextStyle? _appBarTextStyle;
  TextStyle? get appBarTextStyle => _appBarTextStyle;
  Future<bool> setAppBarTextStyle(TextStyle? style) async {
    if (!_customWebViewState.mounted) {
      return Future.value(false);
    } else {
      _appBarTextStyle = style;
      notifyListeners();
      return Future.value(true);
    }
  }

  bool _isHideNavigationBar = false;
  FutureOr<bool> hideNavigationBar() {
    if (!_customWebViewState.mounted) {
      return Future.value(false);
    }
    if (_isHideNavigationBar) {
      return Future.value(true);
    }
    _isHideNavigationBar = true;
    notifyListeners();
    return Future.value(true);
  }

  FutureOr<bool> showNavigationBar() {
    if (!_customWebViewState.mounted) {
      return Future.value(false);
    }
    if (!_isHideNavigationBar) {
      return Future.value(true);
    }
    _isHideNavigationBar = false;
    notifyListeners();
    return Future.value(true);
  }
}
